﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class HealthBar : Mod
{
    public AssetBundle bundle;
    public GameObject healthBar;
    public Slider bar;
    public Text entityText;
    public IEnumerator Start()
    {
        AssetBundleCreateRequest request = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("healthbar.assets"));
        yield return request;
        bundle = request.assetBundle;
        if (RAPI.IsCurrentSceneGame())
        {
            Setup();
        }
        Debug.Log("Mod HealthBar has been loaded!");
    }
    public void OnModUnload()
    {
        Destroy(healthBar);
        bundle.Unload(true);
        Debug.Log("Mod HealthBar has been unloaded!");
    }
    public override void WorldEvent_WorldLoaded()
    {
        Setup();
    }
    public void Setup()
    {
        healthBar = Instantiate(bundle.LoadAsset<GameObject>("HealthBar"), ComponentManager<CanvasHelper>.Value.transform);
        bar = healthBar.GetComponent<Slider>();
        entityText = healthBar.transform.Find("Image").transform.Find("Text").GetComponent<Text>();
        healthBar.name = "HealthBar";
        healthBar.transform.position = new Vector3(960, 1030, 0);
        if (ExtraSettingsAPI_Loaded)
        {
            healthBar.GetComponent<RectTransform>().sizeDelta = new Vector2(ExtraSettingsAPI_GetSliderValue("BarWidth"), ExtraSettingsAPI_GetSliderValue("BarHeight"));
            healthBar.transform.Find("Background").GetComponent<RectTransform>().localPosition = healthBar.transform.Find("Fill Area").GetComponent<RectTransform>().localPosition = new Vector3(0, -(ExtraSettingsAPI_GetSliderValue("BarHeight") - 20) / 2, 0);
            healthBar.GetComponentInChildren<Text>().fontSize = (int)ExtraSettingsAPI_GetSliderValue("FontSize");
            healthBar.transform.Find("Image").GetComponent<RectTransform>().sizeDelta = new Vector2(0, ExtraSettingsAPI_GetSliderValue("FontSize") + 10);
            healthBar.transform.Find("Image").GetComponent<RectTransform>().localPosition = new Vector3(0, 30 + (ExtraSettingsAPI_GetSliderValue("FontSize") - 30) / 2, 0);
            healthBar.GetComponent<RectTransform>().position = new Vector3(ExtraSettingsAPI_GetSliderValue("BarX"), ExtraSettingsAPI_GetSliderValue("BarY"), 0);
        }
    }
    public void FixedUpdate()
    {
        if (RAPI.IsCurrentSceneGame() && healthBar != null && !CanvasHelper.ActiveMenu.Equals(MenuType.PauseMenu))
        {
            RaycastHit hit;
            if (Physics.SphereCast(Camera.main.transform.position, 0.5f, Camera.main.transform.forward, out hit, 40))
            {
                if (hit.collider.gameObject.TryGetComponent(out Network_Entity_Redirect redirect))
                {
                    Network_Entity entity = redirect.entity;
                    healthBar.SetActive(true);
                    entityText.text = $"{NameSimplify(entity.name)}   {entity.stat_health.Value} / {entity.stat_health.Max}";
                    bar.value = entity.stat_health.Value / entity.stat_health.Max;
                } else if (hit.collider.gameObject.GetComponentInParent<Network_Entity_Redirect>() != null)
                {
                    Network_Entity entity = hit.collider.gameObject.GetComponentInParent<Network_Entity_Redirect>().entity;
                    healthBar.SetActive(true);
                    entityText.text = $"{NameSimplify(entity.name)}   {entity.stat_health.Value} / {entity.stat_health.Max}";
                    bar.value = entity.stat_health.Value / entity.stat_health.Max;
                } else
                {
                    healthBar.SetActive(false);
                }
            } else
            {
                healthBar.SetActive(false);
            }
        }
    }
    public string NameSimplify(string name)
    {
        return name.Replace("(Clone)", "").Replace("_", "").Replace("AI", "");
    }

    bool ExtraSettingsAPI_Loaded = false;
    public IEnumerator HealthBarCustomising()
    {
        RectTransform sliderTransform = healthBar.GetComponent<RectTransform>();
        RectTransform barTransform = healthBar.GetComponent<RectTransform>();
        RectTransform bgTransform = healthBar.transform.Find("Background").GetComponent<RectTransform>();
        RectTransform fillTransfrom = healthBar.transform.Find("Fill Area").GetComponent<RectTransform>();
        RectTransform textBgTransform = healthBar.transform.Find("Image").GetComponent<RectTransform>();
        Text text = healthBar.GetComponentInChildren<Text>();
        while (CanvasHelper.ActiveMenu.Equals(MenuType.PauseMenu))
        {
            barTransform.sizeDelta = new Vector2(ExtraSettingsAPI_GetSliderValue("BarWidth"), ExtraSettingsAPI_GetSliderValue("BarHeight"));
            bgTransform.localPosition = fillTransfrom.localPosition = new Vector3(0, -(ExtraSettingsAPI_GetSliderValue("BarHeight") - 20) / 2, 0);
            text.fontSize = (int)ExtraSettingsAPI_GetSliderValue("FontSize");
            textBgTransform.sizeDelta = new Vector2(0, ExtraSettingsAPI_GetSliderValue("FontSize") + 10);
            textBgTransform.localPosition = new Vector3(0, 30 + (ExtraSettingsAPI_GetSliderValue("FontSize") - 30) / 2, 0);
            sliderTransform.position = new Vector3(ExtraSettingsAPI_GetSliderValue("BarX"), ExtraSettingsAPI_GetSliderValue("BarY"), 0);
            yield return new WaitForSeconds(0.02f);
        }
    }
    public void ExtraSettingsAPI_SettingsOpen()
    {
        if (healthBar != null)
        {
            healthBar.SetActive(true);
            bar.value = 1;
            StartCoroutine(HealthBarCustomising());
        }
    }
    public void ExtraSettingsAPI_ButtonPress(string name) // Occurs when a settings button is clicked. "name" is set the the button's name
    {
        if (name == "Reset")
            ExtraSettingsAPI_ResetAllSettings();
    }
    public virtual float ExtraSettingsAPI_GetSliderValue(string SettingName) => 0;
    public virtual void ExtraSettingsAPI_ResetAllSettings() { }
    public virtual void ExtraSettingsAPI_SaveSettings() { }
}